@@ -17,14 +17,18 @@ void HgwPollEvents();
 
 #define kGConfPath "/apps/maemo/drnoksnes"
 #define kGConfRomFile kGConfPath "/" "rom"
-#define kGConfDisableAudio kGConfPath "/" "no_audio"
+#define kGConfSound kGConfPath "/" "sound"
 #define kGConfTurboMode kGConfPath "/" "turbo"
 #define kGConfFrameskip kGConfPath "/" "frameskip"
 #define kGConfTransparency kGConfPath "/" "transparency"
+#define kGConfScaler kGConfPath "/" "scaler"
 #define kGConfDisplayFramerate kGConfPath "/" "display-framerate"
+#define kGConfDisplayControls kGConfPath "/" "display-controls"
 #define kGConfSpeedhacks kGConfPath "/" "speedhacks"
+#define kGConfXSP kGConfPath "/" "xsp"
 #define kGConfMapping kGConfPath "/" "mapping"
 #define kGConfKeysPath kGConfPath "/" "keys"
 
+
 #endif
 
